# coding=utf-8
from django.db import models
from django.contrib.auth.models import User

import hashlib


STATUS = (
    ('created', 'Created'),
    ('succeeded', 'Succeeded'),
    ('failed', 'Failed'),
)

CARD_TYPES = (
    ('1', 'American Express'),
    ('2', 'MasterCard'),
    ('3', 'Visa'),
    ('4', 'Diners'),
    ('5', 'Maestro'),
)


class Transaction(models.Model):
    created_on = models.DateTimeField(auto_now_add=True)
    amount = models.DecimalField(max_digits=12, decimal_places=2)
    status = models.CharField(max_length=10, choices=STATUS)
    user = models.ForeignKey(User, null=True, blank=True)

    pgw_shop_id = models.CharField(max_length=8, blank=True)
    pgw_order_id = models.CharField(max_length=50)
    pgw_amount = models.CharField(max_length=12)
    pgw_authorization_type = models.CharField(max_length=1, blank=True)
    pgw_language = models.CharField(max_length=2, blank=True)
    pgw_success_url = models.CharField(max_length=1000)
    pgw_failure_url = models.CharField(max_length=1000)

    pgw_first_name = models.CharField(max_length=20, blank=True)
    pgw_last_name = models.CharField(max_length=20, blank=True)
    pgw_street = models.CharField(max_length=40, blank=True)
    pgw_city = models.CharField(max_length=20, blank=True)
    pgw_post_code = models.CharField(max_length=9, blank=True)
    pgw_country = models.CharField(max_length=50, blank=True)
    pgw_telephone = models.CharField(max_length=50, blank=True)
    pgw_email = models.CharField(max_length=50, blank=True)
    pgw_disable_installments = models.CharField(max_length=1, blank=True)

    pgw_signature = models.CharField(max_length=128, blank=True)

    pgw_result_code = models.CharField(max_length=4, blank=True)
    pgw_trace_ref = models.CharField(max_length=200, blank=True)
    pgw_transaction_id = models.CharField(max_length=10, blank=True)
    pgw_installments = models.CharField(max_length=10, blank=True)
    pgw_card_type_id = models.CharField(
        max_length=2, choices=CARD_TYPES,
        null=True, blank=True, default=None)

    response_received_on = models.DateTimeField(null=True, blank=True)
    response_signature_valid = models.BooleanField(default=False)

    class Meta:
        db_table = 'htpayway_transactions'

    def __unicode__(self):
        return u'{} {} {} {} {} {:.2f} kn'.format(
            self.pgw_transaction_id, self.pgw_first_name,
            self.pgw_last_name, self.user,
            self.created_on,
            self.amount
        )

    def pgw_data(self):
        return {x.name: getattr(self, x.name)
                for x in self._meta.fields if x.name.startswith('pgw_')}

    def calc_outgoing_signature(self):
        """ Calculates hash signature for outgoing request """

        signature_string = u'authorize-form' + self.pgw_secret_key
        fields = (
            'pgw_shop_id', 'pgw_order_id', 'pgw_amount',
            'pgw_authorization_type', 'pgw_authorization_token', 'pgw_language',
            'pgw_return_method', 'pgw_success_url', 'pgw_failure_url',
            'pgw_first_name', 'pgw_last_name', 'pgw_street', 'pgw_city',
            'pgw_post_code', 'pgw_country', 'pgw_telephone', 'pgw_email',
            'pgw_disable_installments'
        )

        for key in fields:
            value = getattr(self, key, None)
            if value is not None:
                signature_string += value
                signature_string += self.pgw_secret_key
        return hashlib.sha512(signature_string.encode('utf-8')).hexdigest()



from htpayway.views import begin, failure, success
try:
    from django.conf.urls.defaults import patterns, url
except ImportError:
    from django.conf.urls import patterns, url


urlpatterns = patterns(
    '',
    url(r'^begin/(?P<transaction_id>\d+)/$', begin, name='htpayway_begin'),
    url(r'^success/$', success, name='htpayway_success'),
    url(r'^failure/$', failure, name='htpayway_failure'),
)



from django.core import exceptions
from django.core.urlresolvers import reverse
from django.conf import settings
from decimal import Decimal
import hashlib

from .models import Transaction


class PayWay(object):
    pgw_language = ''
    pgw_authorization_type = '1'
    pgw_disable_installments = '1'
    pgw_return_method = 'POST'
    pgw_success_url = None
    pgw_failure_url = None

    def __init__(self, request=None, **pgw_data):
        self.request = request
        for x in pgw_data:
            if x.startswith('pgw_'):
                setattr(self, x, pgw_data[x])

    def pgw_data(self):
        return {x: getattr(self, x)
                for x in dir(self) if x.startswith('pgw_')}

    def calc_incoming_signature(self, data):
        """ Calculates hash signature based on incoming request data """
        signature_string = ''
        for item in data:
            signature_string += item
            signature_string += self.pgw_secret_key
        return hashlib.sha512(signature_string.encode('utf-8')).hexdigest()

    def on_success(self):
        pass

    def on_failure(self):
        pass


def import_callable(callable_path):
    try:
        dot = callable_path.rindex('.')
    except ValueError:
        raise exceptions.ImproperlyConfigured,\
            "%s isn't a callable ([package.[module.]]callable)" % callable_path
    module_name, callable_name = callable_path[:dot], callable_path[dot + 1:]
    try:
        mod = __import__(module_name, {}, {}, [''])
    except ImportError, e:
        raise exceptions.ImproperlyConfigured,\
            'Error importing module %s: "%s"' % (module_name, e)
    try:
        return getattr(mod, callable_name)
    except AttributeError:
        raise exceptions.ImproperlyConfigured,\
            'Module "%s" does not define a "%s" callable' %\
            (module_name, callable_name)


def get_payway_class(htpayway_class=None):
    if htpayway_class is None:
        htpayway_class = settings.HTPAYWAY_CLASS

    if isinstance(htpayway_class, str):
        try:
            return import_callable(htpayway_class)
        except AttributeError as e:
            raise exceptions.ImproperlyConfigured(e)
    else:
        return htpayway_class


def begin_transaction(request, pgw_data, htpayway_class=None):
    """ This should be called from your view
        to create Transaction object.
    """

    user = request.user if request.user.is_authenticated() else None
    amount = pgw_data['amount']

    tx = Transaction()
    tx.status = 'created'
    tx.user = user

    """
    result.pgw_shop_id = pgw_data['pgw_shop_id']
    result.pgw_authorization_type = pgw_data['pgw_authorization_type']
    result.pgw_order_id = pgw_data['pgw_order_id']

    result.pgw_first_name = pgw_data.get('pgw_first_name', ''),
    result.pgw_last_name = pgw_data.get('pgw_last_name', ''),
    result.pgw_street = pgw_data.get('pgw_street', ''),
    result.pgw_city = pgw_data.get('pgw_city', ''),
    result.pgw_post_code = pgw_data.get('pgw_post_code', ''),
    result.pgw_country = pgw_data.get('pgw_country', ''),
    result.pgw_email = pgw_data.get('pgw_email', ''),
    """

    payway = get_payway_class(htpayway_class)()
    payway_data = payway.pgw_data()
    for x in payway_data:
        setattr(tx, x, payway_data[x])

    for x in pgw_data:
        if x.startswith('pgw_'):
            setattr(tx, x, pgw_data[x])

    domain = request.get_host()
    if tx.pgw_success_url is None:
        tx.pgw_success_url = u'http://{}{}'.format(domain, reverse('htpayway_success'))
    if tx.pgw_failure_url is None:
        tx.pgw_failure_url = u'http://{}{}'.format(domain, reverse('htpayway_failure'))
    tx.amount = amount
    tx.pgw_amount = format_amount(amount)

    tx.pgw_signature = tx.calc_outgoing_signature()

    tx.save()
    return tx


def format_amount(amount):
    a = Decimal(amount).quantize(Decimal('0.01'))
    return str(a).replace('.', '')



from django import forms


class SuccessForm(forms.Form):
    pgw_trace_ref = forms.CharField()
    pgw_transaction_id = forms.CharField()
    pgw_order_id = forms.CharField()
    pgw_amount = forms.CharField()
    pgw_installments = forms.CharField()
    pgw_card_type_id = forms.CharField()
    pgw_signature = forms.CharField()


class FailureForm(forms.Form):
    pgw_result_code = forms.CharField()
    pgw_trace_ref = forms.CharField()
    pgw_order_id = forms.CharField()
    pgw_signature = forms.CharField()


class PaymentForm(forms.Form):
    pgw_shop_id = forms.CharField(max_length=8)
    pgw_order_id = forms.CharField(max_length=50)
    pgw_amount = forms.CharField(max_length=12)
    pgw_authorization_type = forms.CharField(max_length=1)
    pgw_language = forms.CharField(max_length=2)
    pgw_return_method = forms.CharField(max_length=4)
    pgw_success_url = forms.CharField(max_length=1000)
    pgw_failure_url = forms.CharField(max_length=1000)
    pgw_first_name = forms.CharField(max_length=20)
    pgw_last_name = forms.CharField(max_length=20)
    pgw_street = forms.CharField(max_length=40)
    pgw_city = forms.CharField(max_length=20)
    pgw_post_code = forms.CharField(max_length=9)
    pgw_country = forms.CharField(max_length=50)
    pgw_telephone = forms.CharField(max_length=50)
    pgw_email = forms.CharField(max_length=50)
    pgw_disable_installments = forms.CharField(max_length=1)
    pgw_signature = forms.CharField(max_length=40)

    def __init__(self, *args, **kwargs):
        super(PaymentForm, self).__init__(*args, **kwargs)
        for f in self.fields:
            self.fields[f].widget = forms.HiddenInput()
            self.fields[f].required = False



from django.contrib import admin
from htpayway.models import Transaction


class TransactionAdmin(admin.ModelAdmin):

    def pgw_name(self):
        return u'{} {}'.format(self.pgw_first_name, self.pgw_last_name)

    list_display = (
        'id', pgw_name, 'user', 'created_on', 'amount',
        'pgw_transaction_id', 'status'
    )
    search_fields = [
        'pgw_first_name', 'pgw_last_name', 'pgw_email',
        'pgw_transaction_id',
    ]
    list_filter = ('created_on', 'status',)
    readonly_fields = ('created_on',)
    fieldsets = (
        ('Order', {
            'fields': (
                'user',
                'created_on',
                'amount',
                'status'
            )
        }),
        ('Response', {
            'fields': (
                ('response_received_on', 'response_signature_valid'),
                ('pgw_trace_ref', 'pgw_transaction_id'),
                ('pgw_installments', 'pgw_card_type_id')
            )
        }),
        ('Pgw data', {
            'fields': (
                'pgw_shop_id',
                'pgw_order_id',
                'pgw_amount',
                'pgw_authorization_type',
                'pgw_first_name',
                'pgw_last_name',
                'pgw_street',
                'pgw_city',
                'pgw_post_code',
                'pgw_country',
                'pgw_email')
        }),
    )


admin.site.register(Transaction, TransactionAdmin)



# -*- coding: utf-8 -*-
from django.test import TestCase, RequestFactory
from django.core import exceptions
from django.contrib.auth.models import AnonymousUser

from .utils import get_payway_class, begin_transaction, format_amount, PayWay
from .models import Transaction

from decimal import Decimal, InvalidOperation


class CustomPayWay(PayWay):
    pgw_shop_id = '123'
    pgw_secret_key = 'secretkey'
    pgw_success_url = u'http://localhost:8000/payway/success/'
    pgw_failure_url = u'http://localhost:8000/payway/failure/'
    pgw_authorization_type = '0'
    pgw_language = 'hr'


class TestImports(TestCase):
    def test_missing_setting_raises(self):
        with self.assertRaises(exceptions.ImproperlyConfigured):
            get_payway_class('')

    def test_get_payment_class_with_string(self):
        pw = get_payway_class('htpayway.tests.CustomPayWay')()
        self.assertEqual(pw.pgw_shop_id, '123')

    def test_get_payment_class_with_class(self):
        pw = get_payway_class(CustomPayWay)()
        self.assertEqual(pw.pgw_shop_id, '123')


class TestPayWay(TestCase):
    def setUp(self):
        self.payway = CustomPayWay()

    def test_create_signature_for_create(self):
        request = RequestFactory().get('/')
        request.user = AnonymousUser()
        transaction = begin_transaction(
            request, {'pgw_order_id': '1', 'amount': '123.00'},
            htpayway_class=CustomPayWay)

        self.assertEqual(
            transaction.pgw_signature,
            'fc424eb91bb260f8364326629b72de6ef7471cf4d09dc3c998657119cd0df2af' +
            '1f313c21108659e87573b0b6525c74f223b0378ab65dbf3e9ffd84697c31b319'
        )

    def test_pgw_arguments_are_initialized(self):
        p = CustomPayWay(pgw_email='a@a.com')
        self.assertEqual(p.pgw_email, 'a@a.com')

    def test_non_pgw_arguments_are_skipped(self):
        p = CustomPayWay(foo=1)
        self.assertFalse(hasattr(p, 'foo'))

    def test_pgw_data_from_model(self):
        p = Transaction(id=1, pgw_transaction_id=2, pgw_amount='300')
        pgw_data = p.pgw_data()

        self.assertNotIn('id', pgw_data)
        self.assertEqual(pgw_data['pgw_transaction_id'], 2)
        self.assertEqual(pgw_data['pgw_amount'], '300')

    def test_pgw_data_from_class(self):
        p = CustomPayWay(pgw_email='a@a.com')

        self.assertEqual(p.pgw_data()['pgw_email'], 'a@a.com')


class TestUtils(TestCase):
    def test_format_amount_raises_on_non_decimal_input(self):
        with self.assertRaises(InvalidOperation):
            format_amount('')

        with self.assertRaises(TypeError):
            format_amount(None)

    def test_format_amount_with_2_decimal_places(self):
        self.assertEqual(format_amount('123.45'), '12345')

    def test_format_amount_with_3_decimal_places(self):
        self.assertEqual(format_amount(Decimal('1000.123')), '100012')



from django.shortcuts import render_to_response, RequestContext, get_object_or_404
from django.views.decorators.csrf import csrf_exempt
from django.http import Http404

from .models import Transaction
from .forms import SuccessForm, FailureForm, PaymentForm
from .utils import get_payway_class

import datetime


def begin(request, transaction_id):
    transaction = get_object_or_404(
        Transaction,
        pk=int(transaction_id), status='created')
    payway = get_payway_class()(request=request,
                                transaction=transaction,
                                **transaction.pgw_data())

    form = PaymentForm(initial=payway.pgw_data())
    return render_to_response('htpayway/begin.html', {'form': form, 'form_url': payway.form_url})


@csrf_exempt
def success(request):
    if request.method == 'POST':
        now = datetime.datetime.now()
        form = SuccessForm(request.POST)
        callback = False
        if form.is_valid():
            payway = get_payway_class()()
            pgw_order_id = form.cleaned_data['pgw_order_id']
            transaction = Transaction.objects.get(pgw_order_id=pgw_order_id, status='created')

            # extract data from response
            pgw_trace_ref = form.cleaned_data['pgw_trace_ref']
            pgw_transaction_id = form.cleaned_data['pgw_transaction_id']
            pgw_amount = form.cleaned_data['pgw_amount']
            pgw_installments = form.cleaned_data['pgw_installments']
            pgw_card_type_id = form.cleaned_data['pgw_card_type_id']

            signature = payway.calc_incoming_signature([
                pgw_trace_ref, pgw_transaction_id, pgw_order_id, pgw_amount,
                pgw_installments, pgw_card_type_id])

            if signature == form.cleaned_data['pgw_signature']:
                transaction.pgw_transaction_id = pgw_transaction_id
                transaction.pgw_trace_ref = pgw_trace_ref
                transaction.pgw_installments = pgw_installments
                transaction.pgw_card_type_id = pgw_card_type_id
                callback = True
                transaction.response_signature_valid = True
                transaction.status = 'succeeded'
            else:
                transaction.response_signature_valid = False

            transaction.response_received_on = now
            transaction.save()

            if callback:
                payway.on_success()
        return render_to_response('htpayway/success.html',
                                  context_instance=RequestContext(request))
    else:
        raise Http404


@csrf_exempt
def failure(request):
    if request.method == 'POST':
        now = datetime.datetime.now()
        form = FailureForm(request.POST)
        callback = False
        if form.is_valid():
            payway = get_payway_class()()
            pgw_order_id = form.cleaned_data['pgw_order_id']
            transaction = Transaction.objects.get(pgw_order_id=pgw_order_id, status='created')

            # extract data from response
            pgw_result_code = form.cleaned_data['pgw_result_code']
            pgw_trace_ref = form.cleaned_data['pgw_trace_ref']
            pgw_signature = payway.calc_incoming_signature(
                [pgw_result_code, pgw_trace_ref, pgw_order_id])

            if pgw_signature == form.cleaned_data['pgw_signature']:
                transaction.pgw_result_code = pgw_result_code
                transaction.pgw_trace_ref = pgw_trace_ref
                callback = True
                transaction.response_signature_valid = True
                transaction.status = 'failed'
            else:
                transaction.response_signature_valid = False

            transaction.pgw_signature = form.cleaned_data['pgw_signature']
            transaction.response_received_on = now
            transaction.save()

            if callback:
                payway.on_failure()
        return render_to_response('htpayway/failure.html',
                                  context_instance=RequestContext(request))
    else:
        raise Http404



import os
from setuptools import setup, find_packages
import htpayway


# allow setup.py to be run from any path
os.chdir(os.path.normpath(os.path.join(os.path.abspath(__file__), os.pardir)))

setup(
    name='django-htpayway',
    url='https://github.com/logithr/djangocms-htpayway',
    author='Sasha Matijasic',
    author_email='sasha@selectnull.com',
    version=htpayway.__version__,
    description='HT PayWay (payment gateway) API integration with Django',
    long_description=open('README.md').read(),
    license='MIT',
    keywords='htpayway ht payway',
    packages=find_packages(),
    include_package_data=True,
    install_requires=[
        'django>=1.3',
    ],
    classifiers=[
        'Development Status :: 4 - Beta',
        'Environment :: Web Environment',
        'Framework :: Django',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 2.7',
        'Topic :: Internet :: WWW/HTTP :: Dynamic Content',
    ],
)
